-- Henge Sphere handler

local AceOO = AceLibrary("AceOO-2.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local Tablet = AceLibrary("Tablet-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Henge")

local sphereTable = {}
local sphereData = {}
local sphereAction = {}

HengeSphere = AceOO.Class()

		---------------------------------
		-- HengeSphere local functions --
		---------------------------------

local function tabletPoint(frame)
	if frame:GetTop() > GetScreenHeight() / 2 then
		local x = frame:GetCenter()
		if x < GetScreenWidth() / 3 then
			return "TOPLEFT", "BOTTOMLEFT"
		elseif x < GetScreenWidth() * 2 / 3 then
			return "TOP", "BOTTOM"
		else
			return "TOPRIGHT", "BOTTOMRIGHT"
		end
	else
		local x = frame:GetCenter()
		if x < GetScreenWidth() / 3 then
			return "BOTTOMLEFT", "TOPLEFT"
		elseif x < GetScreenWidth() * 2 / 3 then
			return "BOTTOM", "TOP"
		else
			return "BOTTOMRIGHT", "TOPRIGHT"
		end
	end
end

		--------------------------------
		-- HengeButton main functions --
		--------------------------------

function HengeSphere:AddData(arg)
	if type(arg) ~= "table" or type(arg.id) ~= "string" then
		Henge:Debug("%s:AddData() - Invalid argument.", self)
		return
	end
	sphereData[arg.id] = arg
end

function HengeSphere:AddAction(aID)
	if type(aID) ~= "string" then
		Henge:Debug("%s:AddAction() - Invalid argument.", self)
		return
	end
	sphereAction[table.maxn(sphereAction) + 1] = aID
end

function HengeSphere:Add(arg)
	if type(arg) ~= "table" or type(arg.id) ~= "string" or
	type(arg.name) ~= "string" then
		Henge:Debug("%s:Add() - Invalid argument.", self)
		return
	end
	sphereTable[arg.id] = HengeSphere:new(arg)
end

function HengeSphere:Get(id)
	if type(id) ~= "string" then
		Henge:Debug("%s:Get() - Invalid argument.", self)
		return nil
	elseif not AceOO.inherits(sphereTable[id], HengeSphere) then
		Henge:Debug("%s:Get() - Sphere '%s' is invalid.", self, id)
		return nil
	end
	return sphereTable[id]
end

function HengeSphere:UpdateAll()
	for k, v in pairs(sphereTable) do
		v:Update()
	end
end

		--------------------------------------------
		-- HengeSphere - handles the info spheres --
		--------------------------------------------

function HengeSphere:ToString()
	return "HengeSphere"
end

function HengeSphere.prototype:init(arg)
	HengeSphere.super.prototype.init(self)

	local defaults = {
		["xpos"] = arg.xpos or 80,
		["ypos"] = arg.ypos or 500,
		["theme"] = arg.theme or "Orange",
		["gfxshow"] = arg.gfxshow or "health",
		["textshow"] = arg.textshow or "health"
	}
	self.id = arg.id
	self.db = Henge:AcquireDBNamespace("sphere"..self.id)
	Henge:RegisterDefaults("sphere"..self.id, "char", defaults)
	self.frame = getglobal("HengeSphere"..self.id)
	if not self.frame then
		self.frame = CreateFrame(
			"Button", "HengeSphere"..self.id,
			UIParent, "HengeSphereTemplate"
		)
	end
	self.frame.hsObj = self
	self.texture = getglobal("HengeSphere"..self.id.."BG")
	self.text = getglobal("HengeSphere"..self.id.."Text")
	self.frame:SetAttribute("type2", "menu")
	self.frame.menu = function() self:OpenMenu() end
	self.dragging = false
	Tablet:Register(self.frame,
		'children', function() self:OnTooltipUpdate() end,
		'point', tabletPoint,
		'data', {}
	)
	local textshow = self:GetOption("textshow")
	local gfxshow = self:GetOption("gfxshow")
	self:SetOption("textshow", nil)
	self:SetOption("gfxshow", nil)
	self:SetShown("text", textshow)
	self:SetShown("gfx", gfxshow)
	self.frame:Show()
	self:RefreshPos()
	self:Update()
end

function HengeSphere.prototype:OnTooltipUpdate()
	Tablet:SetTitle(Henge.info.name.." - "..self.id)
	Tablet:SetHint("RightClick: "..L["Show sphere menu."])
end

function HengeSphere.prototype:RefreshPos()
	local scale = UIParent:GetEffectiveScale()
	self.frame:ClearAllPoints()
	self.frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT",
		self.db.char.xpos / scale,
		self.db.char.ypos / scale
	)
end

function HengeSphere.prototype:SetShown(st, arg)
	if (type(st) ~= "string") or ((st ~= "text") and (st ~= "gfx")) then
		return
	end
	local old = self:GetOption(st.."show")
	self:SetOption(st.."show", arg)
	if old and sphereData[old] then
		if sphereData[old].event and
		Henge:IsBucketEventRegistered(sphereData[old].event) then
			Henge:UnregisterBucketEvent(sphereData[old].event)
		elseif sphereData[old].counter then
			local ot = "text"
			if st == "text" then ot = "gfx" end
			local other = self:GetOption(ot.."show")
			if sphereData[other].counter ~= sphereData[old].counter then
				local c = HengeCounter:Get(sphereData[old].counter)
				if AceOO.inherits(c, HengeCounter) then
					c:RemoveWatch("HengeSphere"..self.id)
				end
			end
		end
	end
	if type(arg) == "string" and type(sphereData[arg]) == "table" then
		if type(sphereData[arg].event) == "string" then
			Henge:RegisterBucketEvent(sphereData[arg].event, 1.5,
				function(arg) self:OnEvent(arg) end)
		elseif sphereData[arg].counter then
			local ot = "text"
			if st == "text" then ot = "gfx" end
			local other = self:GetOption(ot.."show")
			if sphereData[other].counter ~= sphereData[arg].counter then
				local c = HengeCounter:Get(sphereData[arg].counter)
				if AceOO.inherits(c, HengeCounter) then
					c:AddWatch("HengeSphere"..self.id,
						function(count, always)
							self:OnCounterUpdate(count, always)
						end
					)
				end
			end
		end
	end
	self:Update()
end

function HengeSphere.prototype:OnEvent(argt)
	local shouldUpdate = false
	local gopt = self:GetOption("gfxshow")
	local topt = self:GetOption("textshow")
	for arg, _ in pairs(argt) do
		if type(gopt) == "string" and type(sphereData[gopt]) == "table" and
		(type(sphereData[gopt].arg) ~= "string" or 
		arg == sphereData[gopt].arg) then
			shouldUpdate = true
			break
		end
		if type(topt) == "string" and type(sphereData[topt]) == "table" and
		(type(sphereData[topt].arg) ~= "string" or
		arg == sphereData[topt].arg) then
			shouldUpdate = true
			break
		end
	end
	if shouldUpdate then
		self:Update()
		return
	end
end

function HengeSphere.prototype:GetDisplayValues()
	local tval, gval = "0", "0"
	return tval, gval
end

function HengeSphere.prototype:Update()
	local textShow = self:GetOption("textshow")
	local gfxShow = self:GetOption("gfxshow")
	local tpath = Henge:GetThemeBase()..self:GetOption("theme")
	local tval, gval = "", ""
	if sphereData[textShow] then
		local func = nil
		if type(sphereData[textShow].textfunc) == "function" then
			func = sphereData[textShow].textfunc
		elseif type(sphereData[textShow].func) == "function" then
			func = sphereData[textShow].func
		end
		if type(func) == "function" then
			local c, m = func()
			if type(m) == "string" and m == "timer" then
				local c = c or 0
				local min, sec = math.floor(c / 60), mod(c, 60)
				if min > 0 then
					tval = tostring(min).."m"
				end
				if sec > 0.001 then
					tval = tval..tostring(sec).."s"
				end
				if tval == "" then
					tval = "Ready"
				end
			elseif type(m) == "number" then
				local tap = self:GetOption("textpercent")
				if m > 0 and c then
					if tap == true then
						tval = tostring(floor((c / m) * 100))
					else
						tval = tostring(c)
					end
				end
			end
		end
	end
	if sphereData[gfxShow] then
		local func = nil
		if type(sphereData[gfxShow].gfxfunc) == "function" then
			func = sphereData[gfxShow].gfxfunc
		elseif type(sphereData[gfxShow].func) == "function" then
			func = sphereData[gfxShow].func
		end
		if type(func) == "function" then
			local c, m = func()
			if m > 0 and c then
				local p = floor((c / m) * 16)
				if p > 16 then
					p = 16
				elseif p < 0 then
					p = 0
				end
				gval = tpath.."\\Shard"..tostring(p)
			end
		end
	end
	local validgfx = false
	for l = 0, 16 do
		if gval == tpath.."\\Shard"..tostring(l) then
			validgfx = true
		end
	end
	if validgfx == false then
		gval = tpath.."\\Shard0"
	end
	self.texture:SetTexture(gval)
	self.text:SetText(tval)
end

function HengeSphere.prototype:OnCounterUpdate(count, always)
	self:Update()
end

function HengeSphere.prototype:OpenMenu()
	Dewdrop:Open(self.frame,
		'children', function()
			Dewdrop:FeedAceOptionsTable(self:GetMenu())
		end
	)
end

function HengeSphere.prototype:GetMenu()
	local menu = {
		["type"] = 'group',
		["name"] = Henge.info.name,
		["icon"] = Henge.info.icon,
		["args"] = {
			["header"] = {
				["type"] = 'header',
				["name"] = Henge.info.name,
				["icon"] = Henge.info.icon,
				["order"] = 1
			},
			["lock"] = {
				["type"] = 'toggle',
				["name"] = L["Lock Frames"],
				["desc"] = L["Prevent moving of the spheres and buttons."],
				["get"]  = function() return Henge:IsLocked() end,
				["set"]  = function() Henge:ToggleLocked() end,
				["order"] = 6
			},
			["close"] = {
				["type"] = 'execute',
				["name"] = L["Close"],
				["desc"] = L["Close the menu."],
				["func"] = function()
					Dewdrop:Close(1)
				end,
				["order"] = -1
			},
			["theme"] = {
				["type"] = 'text',
				["name"] = L["Theme"],
				["desc"] = L["Set the sphere theme."],
				["get"]  = function()
					return self:GetOption("theme")
				end,
				["set"]  = function(arg)
					self:SetOption("theme", arg)
					self:Update()
				end,
				["validate"] = {
					["Blue"]      = "Blue",
					["Orange"]    = "Orange",
					["Rose"]      = "Rose",
					["Turquoise"] = "Turquoise",
					["Violet"]    = "Violet",
					["X"]         = "X"
				},
				["order"] = 2
			},
			["gfxshow"] = {
				["type"] = 'group',
				["name"] = L["Sphere (graphic)"],
				["desc"] = L["Choose what the sphere graphic displays."],
				["args"] = {},
				["order"] = 3
			},
			["textshow"] = {
				["type"] = 'group',
				["name"] = L["Sphere (text)"],
				["desc"] = L["Choose what the sphere text displays."],
				["args"] = {
					["percent"] = {
						["type"] = 'toggle',
						["name"] = L["Show as percent"],
						["desc"] = L["Toggle text shown as a percentage."],
						["get"]  = function()
							return self:GetOption("textpercent")
						end,
						["set"]  = function(arg)
							if self:GetOption("textpercent") then
								self:SetOption("textpercent", false)
							else
								self:SetOption("textpercent", true)
							end
						end,
						["order"] = 1
					},
					["space"] = {
						["type"] = "header",
						["order"] = 2
					}
				},
				["order"] = 4
			}
		}
	}
	local loop = 5
	for k, v in pairs(sphereData) do
		menu.args.textshow.args[k] = {
			["type"] = 'toggle',
			["name"] = v.name,
			["desc"] = v.desc or v.name,
			["isRadio"] = true,
			["get"]  = function()
				return self:GetOption("textshow") == k
			end,
			["set"]  = function(arg)
				self:SetShown("text", k)
			end,
			["order"] = loop
		}
		menu.args.gfxshow.args[k] = {
			["type"] = 'toggle',
			["name"] = v.name,
			["desc"] = v.desc or v.name,
			["isRadio"] = true,
			["get"]  = function()
				return self:GetOption("gfxshow") == k
			end,
			["set"]  = function(arg)
				self:SetShown("gfx", k)
			end,
			["order"] = loop
		}
		loop = loop + 1
	end
	return menu
end

function HengeSphere.prototype:GetOption(opt)
	if (type(opt) ~= "string") and (type(opt) ~= "number") then
		return nil
	end
	return self.db.char[opt]
end

function HengeSphere.prototype:SetOption(opt, arg)
	if (type(opt) ~= "string") and (type(opt) ~= "number") then
		return false
	end
	self.db.char[opt] = arg
	return true
end

function HengeSphere.prototype:OnDragStart(mbs)
	if not Henge:IsLocked() then
		if mbs == "LeftButton" then
			this:SetScript("OnUpdate", function() self:OnMove() end)
			self.dragging = true
		end
	end
end

function HengeSphere.prototype:OnDragStop()
	if not Henge:IsLocked() then
		if self.dragging then
			this:SetScript("OnUpdate", nil)
			self.dragging = false
		end
	end
end

function HengeSphere.prototype:OnMove()
	local x, y = GetCursorPosition()
	local scale = UIParent:GetEffectiveScale()
	--x, y = x / scale, y / scale
	self:SetOption("xpos", x)
	self:SetOption("ypos", y)
	self:RefreshPos()
end
